<?php
defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'WPCleverWpcbm_Shortcode' ) ) {
	class WPCleverWpcbm_Shortcode {
		function __construct() {
			add_action( 'init', array( $this, 'wpcbm_shortcodes' ) );
		}

		function wpcbm_shortcodes() {
			add_shortcode( 'wpcbm_price', array( $this, 'wpcbm_price_shortcode' ) );
			add_shortcode( 'wpcbm_save_percentage', array( $this, 'wpcbm_save_percentage_shortcode' ) );
			add_shortcode( 'wpcbm_save_amount', array( $this, 'wpcbm_save_amount_shortcode' ) );
			add_shortcode( 'wpcbm_tags', array( $this, 'wpcbm_tags_shortcode' ) );
			add_shortcode( 'wpcbm_categories', array( $this, 'wpcbm_categories_shortcode' ) );
		}

		function wpcbm_price_shortcode( $attrs ) {
			$output = '';

			$attrs = shortcode_atts( array(
				'id' => null
			), $attrs );

			if ( ! $attrs['id'] ) {
				global $product;

				if ( $product ) {
					$output = $product->get_price_html();
				}
			} else {
				if ( $_product = wc_get_product( $attrs['id'] ) ) {
					$output = $_product->get_price_html();
				}
			}

			return $output;
		}

		function wpcbm_save_amount_shortcode( $attrs ) {
			$output = '';

			$attrs = shortcode_atts( array(
				'id' => null
			), $attrs );

			if ( ! $attrs['id'] ) {
				global $product;

				if ( $product && $product->is_on_sale() ) {
					$output = $this->wpcbm_get_save_amount( $product );
				}
			} else {
				if ( ( $_product = wc_get_product( $attrs['id'] ) ) && $_product->is_on_sale() ) {
					$output = $this->wpcbm_get_save_amount( $_product );
				}
			}

			return $output;
		}

		function wpcbm_save_percentage_shortcode( $attrs ) {
			$output = '';

			$attrs = shortcode_atts( array(
				'id' => null
			), $attrs );

			if ( ! $attrs['id'] ) {
				global $product;

				if ( $product && $product->is_on_sale() ) {
					$output = $this->wpcbm_get_save_percentage( $product );
				}
			} else {
				if ( ( $_product = wc_get_product( $attrs['id'] ) ) && $_product->is_on_sale() ) {
					$output = $this->wpcbm_get_save_percentage( $_product );
				}
			}

			return $output;
		}

		function wpcbm_tags_shortcode( $attrs ) {
			$output = '';

			$attrs = shortcode_atts( array(
				'id' => null
			), $attrs );

			if ( ! $attrs['id'] ) {
				global $product;

				if ( $product ) {
					$attrs['id'] = $product->get_id();
				}
			}

			if ( $attrs['id'] ) {
				$output = wc_get_product_tag_list( $attrs['id'] );
			}

			return $output;
		}

		function wpcbm_categories_shortcode( $attrs ) {
			$output = '';

			$attrs = shortcode_atts( array(
				'id' => null
			), $attrs );

			if ( ! $attrs['id'] ) {
				global $product;

				if ( $product ) {
					$attrs['id'] = $product->get_id();
				}
			}

			if ( $attrs['id'] ) {
				$output = wc_get_product_category_list( $attrs['id'] );
			}

			return $output;
		}

		function wpcbm_get_save_percentage( $product ) {
			$output = '';

			if ( $product->get_type() == 'variable' ) {
				$available_variations = $product->get_variation_prices();
				$max_percentage       = 0;

				foreach ( $available_variations['regular_price'] as $key => $regular_price ) {
					$sale_price = $available_variations['sale_price'][ $key ];

					if ( $regular_price && $sale_price < $regular_price ) {
						$percentage = round( ( $regular_price - $sale_price ) * 100 / $regular_price );

						if ( $percentage > $max_percentage ) {
							$max_percentage = $percentage;
						}
					}
				}

				$output = $max_percentage . '%';
			} else {
				$regular_price = $product->get_regular_price();
				$sale_price    = $product->get_sale_price();

				if ( $regular_price && $sale_price ) {
					$output = round( ( $regular_price - $sale_price ) * 100 / $regular_price ) . '%';
				}
			}

			return $output;
		}

		function wpcbm_get_save_amount( $product ) {
			$output = '';

			if ( $product->get_type() == 'variable' ) {
				$available_variations = $product->get_variation_prices();
				$max_amount           = 0;

				foreach ( $available_variations['regular_price'] as $key => $regular_price ) {
					$sale_price = $available_variations['sale_price'][ $key ];

					if ( $regular_price && $sale_price < $regular_price ) {
						$amount = $regular_price - $sale_price;

						if ( $amount > $max_amount ) {
							$max_amount = $amount;
						}
					}
				}

				$output = wc_price( $max_amount );
			} else {
				$regular_price = $product->get_regular_price();
				$sale_price    = $product->get_sale_price();

				if ( $regular_price && $sale_price < $regular_price ) {
					$output = wc_price( $regular_price - $sale_price );
				}
			}

			return $output;
		}
	}

	new WPCleverWpcbm_Shortcode();
}